

class GenericTemplate(object):

    template_type = 'generic'

    def __init__(self, elements):
        if not isinstance(elements, list):
            raise ValueError(
                'elements should be a list of Element'
            )
        self._elements = elements

    @property
    def elements(self):
        if len(self._elements) > 10:
            raise ValueError('Too many elements in the template')
        return self._elements

    def to_dict(self):
        return {
            'template_type': self.template_type,
            'elements': [
                element.to_dict() for element in self.elements
            ]
        }


class ButtonTemplate(object):

    template_type = 'button'

    def __init__(self, text, buttons):
        self.text = text

        if not isinstance(buttons, list):
            raise ValueError(
                'buttons should be a list of Button'
            )
        self.buttons = buttons

    def to_dict(self):
        return {
            'template_type': self.template_type,
            'text': self.text,
            'buttons': [
                button.to_dict() for button in self.buttons
            ]
        }





class Element(object):

    def __init__(self, title, item_url=None,
                 image_url=None, subtitle=None, buttons=None):

        self._title = title
        self.item_url = item_url
        self.image_url = image_url
        self._subtitle = subtitle
        self.buttons = buttons

    @property
    def title(self):
        if len(self._title) > 45:
            raise ValueError(
                'Element.title has more than 45 characters'
            )

    @property
    def subtitle(self):
        if len(self._subtitle) > 80:
            raise ValueError(
                'Element.subtitle has more than 80 characters'
            )

    def to_dict(self):
        return {
            'title': self.title,
            'item_url': self.item_url,
            'image_url': self.image_url,
            'subtitle': self.subtitle,
            'buttons': [
                button.to_dict() for button in self.buttons
            ]
        }


class Button(object):

    def __init__(self, title):
        if len(title) > 20:
            raise ValueError('Button title limit is 20 characters')
        self.title = title

    def to_dict(self):
        serialised = {
            'type': self.button_type,
            'title': self.title
        }
        if self.button_type == 'web_url':
            serialised['url'] = self.url
        elif self.button_type == 'postback':
            serialised['payload'] = self.payload
        return serialised


class WebUrlButton(Button):

    button_type = 'web_url'

    def __init__(self, title, url):
        self.url = url
        super(WebUrlButton, self).__init__(title=title)


class PostbackButton(Button):

    button_type = 'postback'

    def __init__(self, title, payload):
        self.payload = payload
        super(PostbackButton, self).__init__(title=title)



import json


class Message(object):

    def __init__(self, text=None, attachment=None):
        if not text and not attachment:
            raise ValueError()
        self.text = text
        self.attachment = attachment

    def to_dict(self):
        data = {}
        if self.text:
            data['text'] = self.text
        if self.attachment:
            data['attachment'] = self.attachment.to_dict()
        return data


class Recipient(object):

    def __init__(self, recipient_id=None, phone_number=None):
        if not recipient_id and not phone_number:
            raise ValueError('<Recipient> id or phone_number must be set')
        self.recipient_id = recipient_id
        self.phone_number = phone_number

    def to_dict(self):
        if self.recipient_id:
            return {'id': self.recipient_id}
        return {'phone_number': self.phone_number}


class MessageRequest(object):

    NOTIFICATION_TYPE_OPTIONS = (
        'REGULAR', 'SILENT_PUSH', 'NO_PUSH'
    )

    def __init__(self, recipient, message, notification_type=None):
        self.recipient = recipient
        self.message = message
        self._notification_type = notification_type

    @property
    def notification_type(self):
        if self._notification_type:
            if self._notification_type not in self.NOTIFICATION_TYPE_OPTIONS:
                raise ValueError(
                    'notification_type valid options: %s' %
                    str(self.NOTIFICATION_TYPE_OPTIONS)
                )
        return self._notification_type

    def to_dict(self):
        data = {
            'recipient': self.recipient.to_dict(),
            'message': self.message.to_dict()
        }
        if self.notification_type:
            data['notification_type'] = self.notification_type
        return data

    def serialise(self):
        return json.dumps(self.to_dict())



from setuptools import setup

setup(
    name='messengerbot',
    packages=['messengerbot'],
    license='The MIT License (MIT)',
    version='0.1.1',
    description='Python client for FB Messenger Platform Bot',
    long_description=open('README.rst').read(),
    author='Nam Ngo',
    author_email='namngology@gmail.com',
    url='https://github.com/geeknam',
    keywords='bot facebook messenger platform',
    install_requires=['requests']
)




import os


class MessengerException(Exception):
    pass


class MessengerError(object):

    def __init__(self, *args, **kwargs):
        self.__dict__.update(**kwargs)

    def raise_exception(self):
        raise MessengerError(
            self.error_data
        )


class MessengerClient(object):

    GRAPH_API_URL = 'https://graph.facebook.com/v2.6/me/messages'

    def __init__(self, access_token):
        self.access_token = access_token

    def send(self, message):
        params = {
            'access_token': self.access_token
        }
        response = requests.post(
            self.GRAPH_API_URL, params=params,
            data=message.serialise()
        )
        if response.status_code != 200:
            MessengerError(
                **response.json()['error']
            ).raise_exception()
        return response.json()


ENV_KEY = 'MESSENGER_PLATFORM_ACCESS_TOKEN'

if ENV_KEY in os.environ:
    messenger = MessengerClient(
        access_token=os.environ[ENV_KEY]
    )




class Attachment(object):

    def to_dict(self):
        return {
            'type': self.attachment_type,
            'payload': self.payload
        }


class ImageAttachment(Attachment):

    attachment_type = 'image'

    def __init__(self, url):
        self._url = url

    @property
    def payload(self):
        return {
            'url': self._url
        }


class TemplateAttachment(Attachment):

    attachment_type = 'template'

    def __init__(self, template):
        self.template = template

    @property
    def payload(self):
        return self.template.to_dict()
